@@ -20,16 +20,15 @@ Currently, the library supports parameter learning for *discrete* nodes:
 
 
 # %% Libraries
-from pgmpy.estimators import MaximumLikelihoodEstimator, BayesianEstimator  # ParameterEstimator
-from pgmpy.models import BayesianModel
-from bnlearn.bnlearn import adjmat2vec
+from pgmpy.estimators import MaximumLikelihoodEstimator, BayesianEstimator
+from bnlearn.bnlearn import _filter_df
 from bnlearn.bnlearn import to_BayesianModel
 
 
 # %% Sampling from model
 def fit(model, df, methodtype='bayes', verbose=3):
     """Learn the parameters given the DAG and data.
-    
+
     Description
     -----------
     Maximum Likelihood Estimation
@@ -48,7 +47,7 @@ def fit(model, df, methodtype='bayes', verbose=3):
         be done seperately for `10^3 = 1000` parents configurations.
         This makes MLE very fragile and unstable for learning Bayesian Network parameters.
         A way to mitigate MLE's overfitting is *Bayesian Parameter Estimation*.
-    
+
     Bayesian Parameter Estimation
         The Bayesian Parameter Estimator starts with already existing prior CPDs,
         that express our beliefs about the variables *before* the data was observed.
@@ -58,7 +57,7 @@ def fit(model, df, methodtype='bayes', verbose=3):
         to the actual counts before normalization. Unless one wants to encode specific beliefs
         about the distributions of the variables, one commonly chooses uniform priors,
         i.e. ones that deem all states equiprobable.
-    
+
         A very simple prior is the so-called *K2* prior, which simply adds `1` to the count of every single state.
         A somewhat more sensible choice of prior is *BDeu* (Bayesian Dirichlet equivalent uniform prior).
         For BDeu we need to specify an *equivalent sample size* `N` and then the pseudo-counts are
@@ -70,19 +69,17 @@ def fit(model, df, methodtype='bayes', verbose=3):
         Contains a model object with a key 'adjmat' (adjacency matrix).
     df : pd.DataFrame()
         Pandas DataFrame containing the data.
-    methodtype : str, optional
-        strategy for parameter learning.. The default is 'bayes'.
-        'ml' or 'maximumlikelihood' (default) :Learning CPDs using Maximum Likelihood Estimators
-        'bayes' :Bayesian Parameter Estimation
+    methodtype : str, (default: 'bayes')
+        strategy for parameter learning.
+        Options are: 'ml' or 'maximumlikelihood' for learning CPDs using Maximum Likelihood Estimators. or 'bayes' for Bayesian Parameter Estimation.
     verbose : int, optional
         Print progress to screen. The default is 3.
-        0: NONE
-        1: ERROR
-        2: WARNING
-        3: INFO (default)
-        4: DEBUG
-        5: TRACE
-
+            * 0: NONE
+            * 1: ERROR
+            * 2: WARNING
+            * 3: INFO (default)
+            * 4: DEBUG
+            * 5: TRACE
 
     Returns
     -------
@@ -90,51 +87,62 @@ def fit(model, df, methodtype='bayes', verbose=3):
 
     Examples
     --------
-    >>> df = bnlearn.import_example()
-    >>> model = bnlearn.import_DAG('sprinkler', CPD=False)
-    >>> model_update = bnlearn.parameter_learning.fit(model, df)
-    >>> bnlearn.plot(model_update)
+    >>> import bnlearn as bn
+    >>>
+    >>> df = bn.import_example()
+    >>> model = bn.import_DAG('sprinkler', CPD=False)
+    >>>
+    >>> # Parameter learning
+    >>> model_update = bn.parameter_learning.fit(model, df)
+    >>> bn.plot(model_update)
     >>>
     >>> # LOAD BIF FILE
-    >>> model = bnlearn.import_DAG('alarm')
-    >>> df = bnlearn.sampling(model, n=1000)
-    >>> model_update = bnlearn.parameter_learning.fit(model, df)
-    >>> G = bnlearn.plot(model_update)
+    >>> model = bn.import_DAG('alarm')
+    >>> df = bn.sampling(model, n=1000)
+    >>> model_update = bn.parameter_learning.fit(model, df)
+    >>> G = bn.plot(model_update)
 
     """
-    config = dict()
+    config = {}
     config['verbose'] = verbose
     config['method'] = methodtype
     adjmat = model['adjmat']
 
-    if verbose>=3: print('[BNLEARN][PARAMETER LEARNING] Computing parameters using [%s]' %(config['method']))
+    # Check whether all labels in the adjacency matrix are included from the dataframe
+    # adjmat, model = _check_adjmat(model, df)
+    df = _filter_df(adjmat, df, verbose=config['verbose'])
+
+    if config['verbose']>=3: print('[BNLEARN][PARAMETER LEARNING] Computing parameters using [%s]' %(config['method']))
     # Extract model
     if isinstance(model, dict):
         model = model['model']
+
     # Convert to BayesianModel
     if 'BayesianModel' not in str(type(model)):
-        model = to_BayesianModel(adjmat, verbose=verbose)
+        model = to_BayesianModel(adjmat, verbose=config['verbose'])
 
-#    pe = ParameterEstimator(model, df)
-#    print("\n", pe.state_counts('Cloudy'))
-#    print("\n", pe.state_counts('Sprinkler'))
+    # pe = ParameterEstimator(model, df)
+    # print("\n", pe.state_counts('Cloudy'))
+    # print("\n", pe.state_counts('Sprinkler'))
 
     # Learning CPDs using Maximum Likelihood Estimators
     if config['method']=='ml' or config['method']=='maximumlikelihood':
-        mle = MaximumLikelihoodEstimator(model, df)
-        for node in mle.state_names:
-            print(mle.estimate_cpd(node))
+        # mle = MaximumLikelihoodEstimator(model, df)
+        model = MaximumLikelihoodEstimator(model, df)
+        for node in model.state_names:
+            print(model.estimate_cpd(node))
 
     #  Learning CPDs using Bayesian Parameter Estimation
     if config['method']=='bayes':
-        model.fit(df, estimator=BayesianEstimator, prior_type="BDeu", equivalent_sample_size=1000)  # default equivalent_sample_size=5
+        model.fit(df, estimator=BayesianEstimator, prior_type="BDeu", equivalent_sample_size=1000)
 
         for cpd in model.get_cpds():
-            if verbose>=3: print("CPD of {variable}:".format(variable=cpd.variable))
-            if verbose>=3: print(cpd)
-    
+            if config['verbose']>=3: print("CPD of {variable}:".format(variable=cpd.variable))
+            if config['verbose']>=3: print(cpd)
+
     out = {}
     out['model'] = model
     out['adjmat'] = adjmat
+    out['config'] = config
 
     return(out)
